//========================================================================================
//  
//  $File: $
//  
//  Owner: Aleksei Yashin
//  
//  $Author: $
//  
//  $DateTime: $
//  
//  $Revision: $
//  
//  $Change: $
//  
//  Copyright 1997-2003 Adobe Systems Incorporated. All rights reserved.
//  
//  NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance 
//  with the terms of the Adobe license agreement accompanying it.  If you have received
//  this file from a source other than Adobe, then your use, modification, or 
//  distribution of it requires the prior written permission of Adobe.
//  
//========================================================================================

#include "VCPlugInHeaders.h"

// General includes:
#include "MenuDef.fh"
#include "ActionDef.fh"
#include "ActionDefs.h"
#include "AdobeMenuPositions.h"
#include "LocaleIndex.h"
#include "PMLocaleIds.h"
#include "StringTable.fh"
#include "ObjectModelTypes.fh"
#include "ShuksanID.h"
#include "ActionID.h"
#include "CommandID.h"
#include "WorkspaceID.h"
#include "WidgetID.h"
#include "BuildNumber.h"
#include "GenericID.h"



// Project includes:
#include "BrdrID.h"

#ifdef __ODFRC__

/*
// Plugin version definition.
*/
resource PluginVersion (kSDKDefPluginVersionResourceID)
{
    kTargetVersion,
    kBrdrPluginID,
    kSDKDefPlugInMajorVersionNumber, kSDKDefPlugInMinorVersionNumber,
    kSDKDefHostMajorVersionNumber, kSDKDefHostMinorVersionNumber,
    kSDKDefPersistMajorVersionNumber, kSDKDefPersistMinorVersionNumber,
    { kInDesignProduct, kInCopyProduct },
    { kWildFS },
    SDK_DEF_MAKE_VERSIONSTRING(kSDKDefPlugInMajorVersionNumberForResource, kSDKDefPlugInMinorVersionNumberForResource, kSDKDefPlugInStepVersionNumberForResource, kBuildNumber)
};

/*
//  Boss class definition.
*/
resource ClassDescriptionTable(kSDKDefClassDescriptionTableResourceID)
{{{

    /**
     This boss class supports two interfaces:
     IActionComponent and IPMPersist.
    */
    Class
    {
        kBrdrActionComponentBoss,
        kInvalidClass,
        {
            /** Handle the actions from the menu. */
            IID_IACTIONCOMPONENT, kBrdrActionComponentImpl,
            /** Persist the state of the menu across application instantiation.
            Implementation provided by the API.*/
            IID_IPMPERSIST, kPMPersistImpl
        }
    },


  AddIn
    {
        kIntegratorSuiteBoss,
        kInvalidClass,
        {
            IID_IBORDERSUITE, kBorderSuiteASBImpl,
        }
    },

  AddIn
    {
        kLayoutSuiteBoss,
        kInvalidClass,
        {
            IID_IBORDERSUITE, kBorderSuiteLayoutCSBImpl,
        }
    },

}}};

/*
// Implementation definition.
*/
resource FactoryList (kSDKDefFactoryListResourceID)
{
    kImplementationIDSpace,
    {
#include "BrdrFactoryList.h"
    }
};


/*
// Menu definition.
*/
resource MenuDef (kSDKDefMenuResourceID)
{
    {
/*        // The About Plug-ins sub-menu item for this plug-in.
        kBrdrAboutActionID,         // ActionID (kInvalidActionID for positional entries)
        kBrdrAboutMenuPath,         // Menu Path.
        kSDKDefAlphabeticPosition,          // Menu Position.
        kSDKDefIsNotDynamicMenuFlag,        // kSDKDefIsNotDynamicMenuFlag or kSDKDefIsDynamicMenuFlag*/

        kBrdrMenuItem1ActionID,
        kBrdrTargetMenuPath,
        kBrdrMenuItem1MenuItemPosition,
        kSDKDefIsNotDynamicMenuFlag,


    }
};

/*
// Action definition.
*/
resource ActionDef (kSDKDefActionResourceID)
{
    {
/*        kBrdrActionComponentBoss,       // ClassID of boss class that implements the ActionID.
        kBrdrAboutActionID, // ActionID.
        kBrdrAboutMenuKey,  // Sub-menu string.
        kOtherActionArea,               // Area name (see ActionDefs.h).
        kNormalAction,                  // Type of action (see ActionDefs.h).
        kDisableIfLowMem,               // Enabling type (see ActionDefs.h).
        kInvalidInterfaceID,            // Selection InterfaceID this action cares about or kInvalidInterfaceID.
        kSDKDefInvisibleInKBSCEditorFlag, // kSDKDefVisibleInKBSCEditorFlag or kSDKDefInvisibleInKBSCEditorFlag.*/

        kBrdrActionComponentBoss,
        kBrdrMenuItem1ActionID,
        kBrdrMenuItem1MenuItemKey,
        kOtherActionArea,
        kNormalAction,
//        kDisableIfLowMem,
//        kInvalidInterfaceID,
		kDisableIfSelectionDoesNotSupportIID | kCustomEnabling,
		IID_IBORDERSUITE,
        kSDKDefInvisibleInKBSCEditorFlag,



    }
};


/*
// LocaleIndex
// The LocaleIndex should have indicies that point at your
// localizations for each language system that you are
// localized for.
*/
/*
// String LocaleIndex.
*/
resource LocaleIndex (kSDKDefStringsResourceID)
{
    kStringTableRsrcType,
    {
        kWildFS, k_enUS, kSDKDefStringsResourceID + index_enUS
/*        kWildFS, k_enGB, kSDKDefStringsResourceID + index_enUS
        kWildFS, k_deDE, kSDKDefStringsResourceID + index_enUS
        kWildFS, k_frFR, kSDKDefStringsResourceID + index_enUS
        kWildFS, k_esES, kSDKDefStringsResourceID + index_enUS
        kWildFS, k_ptBR, kSDKDefStringsResourceID + index_enUS
        kWildFS, k_svSE, kSDKDefStringsResourceID + index_enUS
        kWildFS, k_daDK, kSDKDefStringsResourceID + index_enUS
        kWildFS, k_nlNL, kSDKDefStringsResourceID + index_enUS
        kWildFS, k_itIT, kSDKDefStringsResourceID + index_enUS
        kWildFS, k_noNO, kSDKDefStringsResourceID + index_enUS
        kWildFS, k_fiFI, kSDKDefStringsResourceID + index_enUS
        kInDesignJapaneseFS, k_jaJP, kSDKDefStringsResourceID + index_jaJP*/
    }
};

resource LocaleIndex (kSDKDefStringsNoTransResourceID)
{
    kStringTableRsrcType,
    {
        kWildFS, k_Wild, kSDKDefStringsNoTransResourceID + index_enUS
    }
};

resource StringTable (kSDKDefStringsNoTransResourceID + index_enUS)
{
    k_enUS,                                 // Locale Id
    kEuropeanMacToWinEncodingConverter,     // Character encoding converter
    {
        // No-Translate strings go here:

    }
};





#endif // __ODFRC__

#include "Brdr_enUS.fr"
#include "Brdr_jaJP.fr"

//  Code generated by DollyXS code generator
