//========================================================================================
//  
//  $File: $
//  
//  Owner: Aleksei Yashin
//  
//  $Author: $
//  
//  $DateTime: $
//  
//  $Revision: $
//  
//  $Change: $
//  
//  Copyright 1997-2003 Adobe Systems Incorporated. All rights reserved.
//  
//  NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance 
//  with the terms of the Adobe license agreement accompanying it.  If you have received
//  this file from a source other than Adobe, then your use, modification, or 
//  distribution of it requires the prior written permission of Adobe.
//  
//========================================================================================

#include "VCPlugInHeaders.h"

// Interface includes:
#include "IActiveContext.h"
#include "ISelectionManager.h"
#include "IActionStateList.h"

// General includes:
#include "CActionComponent.h"
#include "CAlert.h"

// Project includes:
#include "BrdrID.h"
#include "IBrdrSuite.h"
#include "DebugUtils.h"

/** Implements IActionComponent; performs the actions that are executed when the plug-in's
    menu items are selected.

*/
class BrdrActionComponent : public CActionComponent
{
public:
/**
 Constructor.
 @param boss interface ptr from boss object on which this interface is aggregated.
 */
        BrdrActionComponent(IPMUnknown* boss);

        /** The action component should perform the requested action.
            This is where the menu item's action is taken.
            When a menu item is selected, the Menu Manager determines
            which plug-in is responsible for it, and calls its DoAction
            with the ID for the menu item chosen.

            @param actionID identifies the menu item that was selected.
            @param ac active context
            @param mousePoint contains the global mouse location at time of event causing action (e.g. context menus). kInvalidMousePoint if not relevant.
            @param widget contains the widget that invoked this action. May be nil. 
            */
        virtual void DoAction(IActiveContext* ac, ActionID actionID, GSysPoint mousePoint, IPMUnknown* widget);

    private:
        /** Encapsulates functionality for the about menu item. */
        void DoAbout();
        
           /** Encapsulates functionality for the MenuItem1 menu item. */
        void DoMenuItem1(IActiveContext* ac);

        void UpdateActionStates(IActiveContext* ac, IActionStateList* listToUpdate, GSysPoint mousePoint, IPMUnknown* widget);


};

/* CREATE_PMINTERFACE
 Binds the C++ implementation class onto its
 ImplementationID making the C++ code callable by the
 application.
*/
CREATE_PMINTERFACE(BrdrActionComponent, kBrdrActionComponentImpl)

/* BrdrActionComponent Constructor
*/
BrdrActionComponent::BrdrActionComponent(IPMUnknown* boss)
: CActionComponent(boss)
{
}

/* DoAction
*/
void BrdrActionComponent::DoAction(IActiveContext* ac, ActionID actionID, GSysPoint mousePoint, IPMUnknown* widget)
{
    switch (actionID.Get())
    {

        case kBrdrAboutActionID:
        {
            this->DoAbout();
            break;
        }
                    
        
        case kBrdrMenuItem1ActionID:
        {
            this->DoMenuItem1(ac);
            break;
        }



        default:
        {
            break;
        }
    }
}

/* DoAbout
*/
void BrdrActionComponent::DoAbout()
{
    CAlert::ModalAlert
    (
        kBrdrAboutBoxStringKey,             // Alert string
        kOKString,                      // OK button
        kNullString,                        // No second button
        kNullString,                        // No third button
        1,                          // Set OK button to default
        CAlert::eInformationIcon                // Information icon.
    );
}

/* DoMenuItem1
*/
void BrdrActionComponent::DoMenuItem1(IActiveContext* ac)
{
//    CAlert::InformationAlert(kBrdrMenuItem1StringKey);

  do
  {
    TESTNIL(IActiveContext, ac)

    InterfacePtr<IBorderSuite> borderSuite(ac->GetContextSelection(), UseDefaultIID());
    TESTNIL(IBorderSuite, borderSuite)

    if(borderSuite->CanBordering()) borderSuite->DoBordering();
  }
  while(kFalse);

}


void BrdrActionComponent::UpdateActionStates(IActiveContext* ac, IActionStateList* listToUpdate, GSysPoint mousePoint, IPMUnknown* widget)
{
  do
  {
    TESTNIL(IActiveContext, ac)
    TESTNIL(IActionStateList, listToUpdate)

    InterfacePtr<IBorderSuite> borderSuite(ac->GetContextSelection(), UseDefaultIID());
    TESTNIL(IBorderSuite, borderSuite)

    for(int32 i = 0; i < listToUpdate->Length(); i ++)
    {
      ActionID actionID = listToUpdate->GetNthAction(i);
      switch(actionID.Get())
      {
        case kBrdrMenuItem1ActionID:
          listToUpdate->SetNthActionState(i, borderSuite->CanBordering() ? kEnabledAction : kDisabled_Unselected);
          break;

        default:
          break;
      }
    }
  }
  while(kFalse);
}

//  Code generated by DollyXS code generator
