#include "VCPlugInHeaders.h"

// Interface includes:
#include "IDatabase.h"
#include "IDocument.h"
#include "IActiveContext.h"
#include "ITextTarget.h"
#include "ITextModel.h"
#include "ITableUtils.h"
#include "IStrand.h"
#include "IStyleNameTable.h"
#include "IAttributeStrand.h"
#include "IAttrReport.h"
#include "ITableCommands.h"
#include "ISwatchUtils.h"
#include "ITextModelCmds.h"
#include "ITextAttrUID.h"
#include "ITableAttrUID.h"
#include "ILayoutTarget.h"
#include "IHierarchy.h"
#include "ITextFrame.h"
#include "IGeometry.h"
#include "ITextModelCmds.h"
#include "IUIDData.h"
#include "IRangeData.h"
#include "ITableTextContainer.h"
#include "ITableGeometry.h"
#include "IMeasurementSystem.h"
#include "IUnitOfMeasure.h"

// General includes:
#include "RangeData.h"
#include "SDKLayoutHelper.h"
#include "AcquireModalCursor.h"

// Project includes:
#include "BrdrID.h"
#include "IBrdrSuite.h"
#include "DebugUtils.h"

class BorderSuiteLayoutCSB : public CPMUnknown<IBorderSuite>
{
  public:
    BorderSuiteLayoutCSB(IPMUnknown* boss);
    virtual ~BorderSuiteLayoutCSB(void);
    virtual bool16 CanBordering() const;
    virtual ErrorCode DoBordering();

  protected:
    UIDRef GetTextModelUIDRef() const;
    ErrorCode SetTextColor(InterfacePtr<ITextModel>& textModel, TextIndex position, int32 length, const UID& colorUID);
    ErrorCode SetFillColor(InterfacePtr<ITableModel>& tableModel, const GridArea& gridAreaTarget, const UID& colorUID);
    ErrorCode CreateFrame(IDataBase* database, const PMRect& bounds, UIDRef& newFrameUIDRef);
    ErrorCode InsertInline(const UIDRef& storyUIDRef, const PMRect& bounds, const TextIndex& whereTextIndex);
    ErrorCode ChangeToInline(const UIDRef& storyUIDRef, const TextIndex& whereTextIndex, const UIDRef& frameUIDRef);
    PMReal ConvertMillimetersToPoint(const PMReal& val);
};

CREATE_PMINTERFACE(BorderSuiteLayoutCSB, kBorderSuiteLayoutCSBImpl)

BorderSuiteLayoutCSB::BorderSuiteLayoutCSB(IPMUnknown* boss) : CPMUnknown<IBorderSuite>(boss)
{
}

BorderSuiteLayoutCSB::~BorderSuiteLayoutCSB()
{
}

bool16 BorderSuiteLayoutCSB::CanBordering() const
{
  bool16 result = kFalse;

  do
  {
    const UIDRef textModelUIDRef = GetTextModelUIDRef();

    InterfacePtr<ITextModel> textModel(textModelUIDRef, UseDefaultIID());
    TESTNIL(ITextModel, textModel)

    result = kTrue;
  }
  while(kFalse);

  return(result);
}


ErrorCode BorderSuiteLayoutCSB::DoBordering()
{
  ErrorCode status = kFailure;

//  DWORD startTick = GetTickCount();

  do
  {
    AcquireWaitCursor wc;

    const UIDRef textModelUIDRef = GetTextModelUIDRef();
    InterfacePtr<ITextModel> textModel(textModelUIDRef, UseDefaultIID());
    TESTNIL(ITextModel, textModel)

    IDocument* document = gSession->GetActiveContext()->GetContextDocument();
    TESTNIL(IDocument, document)
    IDataBase* database = ::GetDataBase(textModel);
    Utils<ITableUtils> tableUtils;
    TESTNIL(ITableUtils, tableUtils)

    InterfacePtr<IAttributeStrand> paraAttributeStrand(static_cast<IAttributeStrand*>(textModel->QueryStrand(kParaAttrStrandBoss, IID_IATTRIBUTESTRAND)));
    TESTNIL(IAttributeStrand, paraAttributeStrand)

    InterfacePtr<IStrand> paraStrand(paraAttributeStrand, UseDefaultIID());
    TESTNIL(IStrand, paraStrand)

    InterfacePtr<const IStyleNameTable> styleNameTable(document->GetDocWorkSpace(), IID_IPARASTYLENAMETABLE);
    TESTNIL(IStyleNameTable, styleNameTable)

    PMString paraStyle1Name("ramka");
    paraStyle1Name.Translate();
    const UID paraStyle1UID = styleNameTable->FindByName(paraStyle1Name);

    PMString paraStyle2Name("inv");
    paraStyle2Name.Translate();    
    const UID paraStyle2UID = styleNameTable->FindByName(paraStyle2Name);

    PMString paraStyle3Name("photo");
    paraStyle3Name.Translate();    
    const UID paraStyle3UID = styleNameTable->FindByName(paraStyle3Name);

    const UID swatch1UID = Utils<ISwatchUtils>()->GetNamedSwatch(PMString("Black"), database);
    const UID swatch2UID = Utils<ISwatchUtils>()->GetNamedSwatch(PMString("Paper"), database);

    TextIndex position = 0;
    int32 length = 0;
    const int32 totalLength = textModel->TotalLength();
    
    while(length < totalLength)
    {
      bool paraChanged = false;
      const int32 paraLength = paraStrand->GetRunLength(position);
      int32 tmp = 0;
      const UID paraStyleUID = paraAttributeStrand->GetStyleUID(position, &tmp);
      if(paraStyleUID != kInvalidUID)
      {
        const Text::StoryRange textRange(position, position + paraLength - 1);
        if(paraStyleUID == paraStyle1UID)
        {
          SetTextColor(textModel, position, paraLength, swatch1UID);
          const UIDRef tableUIDRef = tableUtils->ConvertTextToTable(textModel, textRange);
          InterfacePtr<ITableModel> tableModel(tableUIDRef, UseDefaultIID());
          TESTNIL(ITableModel, tableModel)
          const GridArea cell(0, 0, 1, 1);
          SetFillColor(tableModel, cell, swatch2UID);
          paraChanged = true;
        }
        else if(paraStyleUID == paraStyle2UID)
        {
          SetTextColor(textModel, position, paraLength, swatch2UID);
          const UIDRef tableUIDRef = tableUtils->ConvertTextToTable(textModel, textRange);
          InterfacePtr<ITableModel> tableModel(tableUIDRef, UseDefaultIID());
          TESTNIL(ITableModel, tableModel)
          const GridArea cell(0, 0, 1, 1);
          SetFillColor(tableModel, cell, swatch1UID);
          paraChanged = true;
        }
        else if(paraStyleUID == paraStyle3UID)
        {
          const UIDRef tableUIDRef = tableUtils->ConvertTextToTable(textModel, textRange);
          InterfacePtr<ITableModel> tableModel(tableUIDRef, UseDefaultIID());
          TESTNIL(ITableModel, tableModel)

          InterfacePtr<ITableGeometry> tableGeometry(tableModel, UseDefaultIID());
          TESTNIL(ITableGeometry, tableGeometry)
          const PMReal colWidth = tableGeometry->GetColWidths(0);

          InterfacePtr<ITableTextContainer> tableTextContainer(tableModel, UseDefaultIID());
          TESTNIL(ITableTextContainer, tableTextContainer);

          InterfacePtr<ITextModel> textModelInTable(tableTextContainer->QueryTextModel());
          TESTNIL(ITextModel, textModelInTable)

          GridAddress gridAddress(0, 0);
          const GridID gridID = tableModel->GetGridID(gridAddress);
          const UIDRef tableRef(::GetUIDRef(tableModel));
          TextIndex threadStart, threadLength;
          const bool16 didFindThread = textModelInTable->FindStoryThread(tableRef.GetUID(), gridID, &threadStart, &threadLength);
          TESTERROR(FindStoryThread, didFindThread == kFalse)

          const PMReal frameHeight = 23; // 
          const PMReal frameWidth = colWidth - ConvertMillimetersToPoint(2);
          const PMRect bounds(0, 0, frameWidth, ConvertMillimetersToPoint(frameHeight));
          RangeData range(threadStart, threadStart);
          UIDRef storyUIDRef = ::GetUIDRef(textModel);
          InsertInline(storyUIDRef, bounds, range.Start(nil));

          paraChanged = true;
        }
      }
      position += paraChanged ? 2 : paraLength;
      length += paraLength;
    }

    status = kSuccess;
  }
  while(kFalse);

//  DWORD endTick = GetTickCount();
//  int msecWork = endTick - startTick;
//  int secWork = msecWork / 1000;
//  float minWork = float(secWork) / 60.;
//  char str[100];
//  sprintf(str, "%ld s = %.1f min", secWork, minWork);
//  CAlert::InformationAlert(str);
  CAlert::InformationAlert("OK");

  return(status);
}

UIDRef BorderSuiteLayoutCSB::GetTextModelUIDRef() const
{
  UIDRef textModelUIDRef(nil, kInvalidUID);

  do
  {
    InterfacePtr<ILayoutTarget> layoutTarget(this, UseDefaultIID());
    if(layoutTarget)
    {
      // get selected page items...
      const bool16 stripStandoffs = kTrue;
      const UIDList itemList(layoutTarget->GetUIDList(stripStandoffs));
      UIDList textFrameList(itemList.GetDataBase());
        
      // collect all text frames from page items...
      const int32 itemCount = itemList.Length();
      for(int32 i = 0; i < itemCount; ++i)
      {
        InterfacePtr<IHierarchy>itemHier(itemList.GetRef(i), UseDefaultIID());
        if(itemHier)
        {
          UIDList textFrameList(itemList.GetDataBase());
          itemHier->GetDescendents(&textFrameList, IID_ITEXTFRAME);
          if(textFrameList.Length())
          {
            const UIDRef& textFrameRef = textFrameList.GetRef(0);
            InterfacePtr<const ITextFrame> textFrame(textFrameRef, UseDefaultIID());
            TESTNIL(ITextFrame, textFrame)
            textModelUIDRef = UIDRef(::GetDataBase(textFrame), textFrame->GetTextModelUID());
            break;
          }
        }
      }
    }
  }
  while(kFalse);
  return(textModelUIDRef);
}

ErrorCode BorderSuiteLayoutCSB::SetTextColor(InterfacePtr<ITextModel>& textModel, TextIndex position, int32 length, const UID& colorUID)
{
  ErrorCode status = kFailure;
    
  do
  {
    TESTERROR(colorID, colorUID == kInvalidUID)

    InterfacePtr<ITextAttrUID> textColorUID(::CreateObject2<ITextAttrUID>(kTextAttrColorBoss));
    TESTNIL(ITextAttrColor, textColorUID)
    textColorUID->Set(colorUID);

    InterfacePtr<IAttrReport> attrReport(textColorUID, UseDefaultIID());
    TESTNIL(IAttrReport, attrReport)

    K2::shared_ptr<AttributeBossList> attrList(new AttributeBossList);
    attrList->ApplyAttribute(attrReport);

    InterfacePtr<ITextModelCmds> textModelCmds(textModel, UseDefaultIID());
    TESTNIL(ITextModelCmds, textModelCmds)
    InterfacePtr<ICommand> applyCmd(textModelCmds->ApplyCmd(position, length, attrList, kParaAttrStrandBoss));
    TESTNIL(ICommand, applyCmd)
    status = CmdUtils::ProcessCommand(applyCmd);
    TESTCMD(ITextModelCmds, status)
  }
  while(kFalse);

  return(status);
}


ErrorCode BorderSuiteLayoutCSB::SetFillColor(InterfacePtr<ITableModel>& tableModel, const GridArea& gridAreaTarget, const UID& colorUID)
{
  ErrorCode status = kFailure;
    
  do
  {
    TESTERROR(colorID, colorUID == kInvalidUID)

    InterfacePtr<ITableCommands> tableCommands(tableModel, UseDefaultIID());
    TESTNIL(ITableCommands, tableCommands)

    InterfacePtr<ITableAttrUID> tableAttrUID(::CreateObject2<ITableAttrUID>(kCellAttrFillColorBoss));
    TESTNIL(ITableAttrUID, tableAttrUID)
    tableAttrUID->Set(colorUID);

    AttributeBossList attrList;
    attrList.ApplyAttribute(tableAttrUID);
    status = tableCommands->ApplyCellOverrides(gridAreaTarget, &attrList);
    TESTERROR(ApplyCellOverrides, status)
  }
  while(kFalse);

  return(status);
}

ErrorCode BorderSuiteLayoutCSB::InsertInline(const UIDRef& storyUIDRef, const PMRect& bounds, const TextIndex& whereTextIndex)
{
  ErrorCode status = kFailure;
  CmdUtils::SequencePtr cmdSeq(status);
  do
  {
    UIDRef newFrameUIDRef;
    status = CreateFrame(storyUIDRef.GetDataBase(), bounds, newFrameUIDRef);
    TESTERROR(CreateFrame, status);

    status = ChangeToInline(storyUIDRef, whereTextIndex, newFrameUIDRef);
    TESTERROR(ChangeToInline, status);
  }
  while(false);
  cmdSeq.SetState(status);
  return(status);
}

ErrorCode BorderSuiteLayoutCSB::CreateFrame(IDataBase* database, const PMRect& bounds, UIDRef& newFrameUIDRef)
{
  SDKLayoutHelper layoutHelper;
  newFrameUIDRef = layoutHelper.CreateRectangleFrame(UIDRef(database, kInvalidUID), bounds);
  return(newFrameUIDRef ? kSuccess : kFailure);
}

ErrorCode BorderSuiteLayoutCSB::ChangeToInline(const UIDRef& storyUIDRef, const TextIndex& whereTextIndex, const UIDRef& frameUIDRef)
{
  ErrorCode status = kFailure;
  do
  {
    InterfacePtr<ITextModel> textModel(storyUIDRef, UseDefaultIID());
    TESTNIL(ITextModel, textModel)

    InterfacePtr<IGeometry> pageItemGeometry(frameUIDRef, UseDefaultIID());
    TESTNIL(IGEOMETRY, pageItemGeometry)

    // Insert character in text flow to anchor the inline.
    K2::shared_ptr<WideString> insertMe(new WideString);

    insertMe->Append(kTextChar_Inline);
    InterfacePtr<ITextModelCmds> textModelCmds(textModel, UseDefaultIID());
    InterfacePtr<ICommand> insertTextCmd(textModelCmds->InsertCmd(whereTextIndex, insertMe));
    TESTNIL(ICommand, insertTextCmd)

    status = CmdUtils::ProcessCommand(insertTextCmd);
    TESTERROR(textModelCmds->InsertCmd, status)

    // Change the page item into an inline.
    InterfacePtr<ICommand> changeILGCmd(CmdUtils::CreateCommand(kChangeILGCmdBoss));
    TESTNIL(ICommand, changeILGCmd)

    InterfacePtr<IRangeData> rangeData(changeILGCmd, UseDefaultIID());
    TESTNIL(IRangeData, rangeData)
    rangeData->Set(whereTextIndex, whereTextIndex);

    InterfacePtr<IUIDData> ilgUIDData(changeILGCmd, UseDefaultIID());
    TESTNIL(IUIDData, ilgUIDData)
    ilgUIDData->Set(frameUIDRef);

    changeILGCmd->SetItemList(UIDList(textModel));
    status = CmdUtils::ProcessCommand(changeILGCmd);
    TESTERROR(changeILGCmd, status);
  }
  while(kFalse);
  return(status);
}

PMReal BorderSuiteLayoutCSB::ConvertMillimetersToPoint(const PMReal& val)
{
  InterfacePtr<IMeasurementSystem> ms(gSession, UseDefaultIID());
  InterfacePtr<IUnitOfMeasure> currentUnit(ms->QueryUnitOfMeasure(ms->Location(kMillimetersBoss)));

  return(currentUnit->UnitsToPoints(val));
}
